/**
 * Legato is a configurable, lightweight web mapping client that can be
 * easily embedded into web pages and portals, CMS and individual web
 * applications. Legato is implemented in JavaScript and based on the
 * popular open source library OpenLayers.
 *
 * Copyright (C) 2010  disy Informationssysteme GmbH, http://www.disy.net
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.disy.legato.tools.gml.servlet;

import java.io.IOException;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.bind.Unmarshaller;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import net.disy.legato.tools.feature.Constants;

public abstract class AbstractGeometryServlet extends HttpServlet {

	private static final long serialVersionUID = 1L;

	private final JAXBContext context;

	public AbstractGeometryServlet() {
		try {
			this.context = JAXBContext.newInstance(Constants.CONTEXT_PATH);
		} catch (JAXBException jaxbex) {
			throw new ExceptionInInitializerError(jaxbex);
		}
	}

	public abstract Object getFeatureCollection(HttpServletRequest request,
			HttpServletResponse response);

	public abstract void setFeatureCollection(HttpServletRequest request,
			HttpServletResponse response, Object featureCollection);

	protected Marshaller getMarshaller() throws Exception {
		Marshaller marshaller = context.createMarshaller();
		marshaller.setProperty("com.sun.xml.bind.namespacePrefixMapper",
				Constants.NAMESPACE_PREFIX_MAPPER);
		marshaller.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE);
		return marshaller;
	}

	protected Unmarshaller getUnmarshaller() throws Exception {
		return context.createUnmarshaller();
	}

	@Override
	protected void doGet(HttpServletRequest request,
			HttpServletResponse response) throws ServletException, IOException {
		try {
			response.setHeader("PRAGMA", "NO-CACHE");
			response.setHeader("CACHE-CONTROL", "NO-CACHE");
			response.setHeader("EXPIRES", "-1");
			final Object featureCollection = getFeatureCollection(request,
					response);

			getMarshaller().marshal(featureCollection,
					new StreamResult(response.getWriter()));
		} catch (Exception ex) {
			throw new ServletException(ex);
		}
	}

	@Override
	protected void doPost(HttpServletRequest request,
			HttpServletResponse response) throws ServletException, IOException {
		try {
			Object featureCollection = getUnmarshaller().unmarshal(
					new StreamSource(request.getReader()));
			setFeatureCollection(request, response, featureCollection);
		} catch (Exception ex) {
			throw new ServletException(ex);
		}
	}
}
