/**
 * Legato is a configurable, lightweight web mapping client that can be
 * easily embedded into web pages and portals, CMS and individual web
 * applications. Legato is implemented in JavaScript and based on the
 * popular open source library OpenLayers.
 *
 * Copyright (C) 2010  disy Informationssysteme GmbH, http://www.disy.net
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.disy.legato.tools.jts;

import java.text.MessageFormat;

import org.directwebremoting.extend.MarshallException;

import com.vividsolutions.jts.geom.Coordinate;
import com.vividsolutions.jts.geom.Point;

public class PointConverter extends AbstractGeometryConverter<Point, double[]> {

  @Override
  protected String getGeometryType() {
    return "Point";
  }

  @Override
  protected Class<double[]> getCoordinatesType() {
    return double[].class;
  }

  @Override
  public double[] createCoordinates(Point point) throws MarshallException {

    if (point == null) {
      return null;
    }
    else if (point.isEmpty()) {
      return new double[0];
    }
    else {
      final double x = point.getCoordinate().x;
      final double y = point.getCoordinate().y;
      final double z = point.getCoordinate().z;

      if (Double.isNaN(z)) {
        return new double[]{ x, y };
      }
      else {
        return new double[]{ x, y, z };
      }
    }
  }

  @Override
  public Point createGeometry(double[] coordinates) throws MarshallException {

    if (coordinates == null) {
      return null;
    }
    else if (coordinates.length == 2) {
      return getGeometryFactory().createPoint(new Coordinate(coordinates[0], coordinates[1]));
    }
    else if (coordinates.length == 3) {
      return getGeometryFactory().createPoint(
          new Coordinate(coordinates[0], coordinates[1], coordinates[2]));
    }
    else {
      throw new MarshallException(Point.class, MessageFormat.format(
          "Incorrect number of coordinate components [{0}].", //$NON-NLS-1$
          coordinates.length));
    }
  }

}
