/**
 * Legato is a configurable, lightweight web mapping client that can be
 * easily embedded into web pages and portals, CMS and individual web
 * applications. Legato is implemented in JavaScript and based on the
 * popular open source library OpenLayers.
 *
 * Copyright (C) 2010  disy Informationssysteme GmbH, http://www.disy.net
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

// Copyright (c) 2008 by disy Informationssysteme GmbH
package net.disy.legato.tools.servlet;

import java.io.IOException;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;

// NOT_PUBLISHED
public class PauseFilter implements Filter {

  private static final String PAUSE_PARAMETER_NAME = "PAUSE"; //$NON-NLS-1$

  private static final int PAUSE_MAX_VALUE = 100 * 1000;

  @Override
  public void destroy() {
    // Nothing to do
  }

  @Override
  public void doFilter(
      final ServletRequest request,
      final ServletResponse response,
      final FilterChain filterChain) throws ServletException, IOException {

    final String pauseValue = request.getParameter(PAUSE_PARAMETER_NAME);

    if (pauseValue != null) {
      try {
        final int pause = Integer.valueOf(pauseValue).intValue();
        if (pause < PAUSE_MAX_VALUE) {
          try {
            Thread.sleep(pause);
          }
          catch (final InterruptedException ignored) {
            // Nothing to do
          }
        }
        else {
          throw new ServletException("The value of the [PAUSE] parameter [" //$NON-NLS-1$
              + pause
              + "] exceeds the maximum allowed value [" //$NON-NLS-1$
              + PAUSE_MAX_VALUE
              + "]."); //$NON-NLS-1$
        }
      }
      catch (final NumberFormatException nfe) {
        throw new ServletException("The value of the [PAUSE] parameter [" //$NON-NLS-1$
            + pauseValue
            + "] could not be paused to an integer."); //$NON-NLS-1$
      }
    }
    filterChain.doFilter(request, response);
  }

  public void init(final FilterConfig arg0) {
    // Nothing to do
  }

}
